@@ -8,9 +8,15 @@ class EmptyDie(icepool.die.base.BaseDie):
     """ Die with no outcomes. """
     
     def ndim(self):
-        return 'empty'
+        return icepool.Empty
     
     def __init__(self):
+        """ Constructor.
+        
+        Dice should not be constructed directly;
+        instead, use one of the methods defined in `icepool.die.func` 
+        (which are imported into the top-level `icepool` module).
+        """
         self._data = Counts({})
         
     def _unary_op(self, op, *args, **kwargs):
